// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/**********************************
* ComputeAverage
* Calculates the weighted average and
* standard deviation 
**********************************/
#include "OmSourceNumerics.h"
#include <cmath>
#include <qspinbox.h> 

void ComputeAverageMagnitude(float* magnitude, float* error, 
              int size, float& mean, float& standardDeviation)
{    
       
    bool useWeights;
    int i;
      
    if (size == 1) 
    {
        mean = magnitude[0];
	standardDeviation = error[0];
	return;
    }


    double* flux = new double[size];
    double* fluxError = new double[size];

    int nData = -1;
    for (i = 0; i <  size; i++)
    {
        if (magnitude[i] >= 0.0 && magnitude[i] < 100.0)
        {
	    nData = nData + 1; 
            flux[nData] = pow(10.0, (-0.4 * magnitude[i]));                 
            fluxError[nData] = flux[nData] * error[i];
	}
    }   
        
    if (++nData < 2) 
    {
        mean = magnitude[0];
        standardDeviation = error[0];
        delete [] flux;
        delete [] fluxError;
        return;
    }
   
    useWeights = TRUE;
     
    double sum1 = 0.0;
    double sum2 = 0.0;
    double sum3 = 0.0;
 
    for( i = 0; i < nData; i++)
    {
       
        if (fluxError[i] <= 0.0)
	{
	    useWeights = FALSE;
            sum1 = double(size);
            sum3 = double(size);
            break;
        }
    } 
    
    for (i = 0; i < nData; i++)
    { 
       
        if (useWeights)
        {
	    double weight = 1.0 / (fluxError[i] * fluxError[i]);
	    sum1 += weight;
	    sum2 += flux[i] * weight;
	    sum3 += weight * weight;
        }
        else
        {
	    sum2 += flux[i];
          
        }

    }


    mean = sum2 / sum1;  // The mean

    sum2 = 0.0;

    for (i = 0; i < nData; i++)
    {
       
   
        double difference = flux[i] - mean;       
        if (useWeights) 
	{
	    double weight = 1.0 / (fluxError[i] * fluxError[i]);
            sum2 += weight * difference * difference;
        }
        else
	    sum2 += difference * difference;
       
    }

    double d;

    if (useWeights) 
        d = sum1 - sum3 / sum1;
    else
        d = float(size - 1); 
    
    standardDeviation = sqrt(sum2 / d);  //  The standard deviation
    delete [] flux;
    delete [] fluxError;
    
    standardDeviation = standardDeviation / mean;  
    mean = -2.5 * log10(mean);
}



